"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict
from typing_extensions import NotRequired, TypedDict


class CybersourceAntiFraudOptionsTypedDict(TypedDict):
    merchant_defined_data: NotRequired[Nullable[Dict[str, str]]]
    r"""A list of merchant defined data to be passed to the Cybersource Decision Manager API. Each key needs to be a numeric string."""
    meta_key_merchant_id: NotRequired[Nullable[str]]
    r"""The merchant ID to use for this transaction. This requires a meta key to be set up for use with Cybersource Decision Manager, and this overrides the connector configuration."""
    shipping_method: NotRequired[Nullable[str]]
    r"""The shipping method for this transaction."""


class CybersourceAntiFraudOptions(BaseModel):
    merchant_defined_data: OptionalNullable[Dict[str, str]] = UNSET
    r"""A list of merchant defined data to be passed to the Cybersource Decision Manager API. Each key needs to be a numeric string."""

    meta_key_merchant_id: OptionalNullable[str] = UNSET
    r"""The merchant ID to use for this transaction. This requires a meta key to be set up for use with Cybersource Decision Manager, and this overrides the connector configuration."""

    shipping_method: OptionalNullable[str] = UNSET
    r"""The shipping method for this transaction."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "merchant_defined_data",
            "meta_key_merchant_id",
            "shipping_method",
        ]
        nullable_fields = [
            "merchant_defined_data",
            "meta_key_merchant_id",
            "shipping_method",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ReportExecutionURLGenerateTypedDict(TypedDict):
    expires_in: NotRequired[int]
    r"""The URL expiration time, in minutes."""


class ReportExecutionURLGenerate(BaseModel):
    expires_in: Optional[int] = 5
    r"""The URL expiration time, in minutes."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .airlineleg import AirlineLeg, AirlineLegTypedDict
from .airlinepassenger import AirlinePassenger, AirlinePassengerTypedDict
from datetime import datetime
from gr4vy.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from gr4vy.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import List, Literal, Union
from typing_extensions import Annotated, NotRequired, TypedDict


TicketDeliveryMethod = Union[
    Literal[
        "electronic",
        "other",
    ],
    UnrecognizedStr,
]


class AirlineTypedDict(TypedDict):
    r"""Information about an airline travel."""

    booking_code: NotRequired[Nullable[str]]
    r"""The unique identifier of the reservation in the global distribution system."""
    is_cardholder_traveling: NotRequired[Nullable[bool]]
    r"""Indicates whether the cardholder is traveling."""
    issued_address: NotRequired[Nullable[str]]
    r"""The address of the place/agency that issued the ticket."""
    issued_at: NotRequired[Nullable[datetime]]
    r"""The date that the ticket was last issued in the airline reservation system."""
    issuing_carrier_code: NotRequired[Nullable[str]]
    r"""For airline aggregators, three-character IATA code of the airline issuing the ticket."""
    issuing_carrier_name: NotRequired[Nullable[str]]
    r"""For airline aggregators, name of the airline issuing the ticket."""
    issuing_iata_designator: NotRequired[Nullable[str]]
    r"""For airline aggregators, two-character IATA code of the airline issuing the ticket."""
    issuing_icao_code: NotRequired[Nullable[str]]
    r"""For airline aggregators, three-character ICAO code of the airline issuing the ticket."""
    legs: NotRequired[Nullable[List[AirlineLegTypedDict]]]
    r"""An array of separate trip segments. Each leg contains detailed itinerary information."""
    passenger_name_record: NotRequired[Nullable[str]]
    r"""The Passenger Name Record (PNR) in the airline reservation system."""
    passengers: NotRequired[Nullable[List[AirlinePassengerTypedDict]]]
    r"""An array of the travelling passengers."""
    reservation_system: NotRequired[Nullable[str]]
    r"""The name of the reservation system."""
    restricted_ticket: NotRequired[Nullable[bool]]
    r"""Indicates whether the ticket is restricted (refundable)."""
    ticket_delivery_method: NotRequired[Nullable[TicketDeliveryMethod]]
    r"""The delivery method of the ticket."""
    ticket_number: NotRequired[Nullable[str]]
    r"""The airline's unique ticket number."""
    travel_agency_code: NotRequired[Nullable[str]]
    r"""The IATA travel agency code."""
    travel_agency_invoice_number: NotRequired[Nullable[str]]
    r"""The reference number of the invoice that was issued by the travel agency."""
    travel_agency_name: NotRequired[Nullable[str]]
    r"""The name of the travel agency."""
    travel_agency_plan_name: NotRequired[Nullable[str]]
    r"""The name of the travel agency plan."""


class Airline(BaseModel):
    r"""Information about an airline travel."""

    booking_code: OptionalNullable[str] = UNSET
    r"""The unique identifier of the reservation in the global distribution system."""

    is_cardholder_traveling: OptionalNullable[bool] = UNSET
    r"""Indicates whether the cardholder is traveling."""

    issued_address: OptionalNullable[str] = UNSET
    r"""The address of the place/agency that issued the ticket."""

    issued_at: OptionalNullable[datetime] = UNSET
    r"""The date that the ticket was last issued in the airline reservation system."""

    issuing_carrier_code: OptionalNullable[str] = UNSET
    r"""For airline aggregators, three-character IATA code of the airline issuing the ticket."""

    issuing_carrier_name: OptionalNullable[str] = UNSET
    r"""For airline aggregators, name of the airline issuing the ticket."""

    issuing_iata_designator: OptionalNullable[str] = UNSET
    r"""For airline aggregators, two-character IATA code of the airline issuing the ticket."""

    issuing_icao_code: OptionalNullable[str] = UNSET
    r"""For airline aggregators, three-character ICAO code of the airline issuing the ticket."""

    legs: OptionalNullable[List[AirlineLeg]] = UNSET
    r"""An array of separate trip segments. Each leg contains detailed itinerary information."""

    passenger_name_record: OptionalNullable[str] = UNSET
    r"""The Passenger Name Record (PNR) in the airline reservation system."""

    passengers: OptionalNullable[List[AirlinePassenger]] = UNSET
    r"""An array of the travelling passengers."""

    reservation_system: OptionalNullable[str] = UNSET
    r"""The name of the reservation system."""

    restricted_ticket: OptionalNullable[bool] = UNSET
    r"""Indicates whether the ticket is restricted (refundable)."""

    ticket_delivery_method: Annotated[
        OptionalNullable[TicketDeliveryMethod],
        PlainValidator(validate_open_enum(False)),
    ] = UNSET
    r"""The delivery method of the ticket."""

    ticket_number: OptionalNullable[str] = UNSET
    r"""The airline's unique ticket number."""

    travel_agency_code: OptionalNullable[str] = UNSET
    r"""The IATA travel agency code."""

    travel_agency_invoice_number: OptionalNullable[str] = UNSET
    r"""The reference number of the invoice that was issued by the travel agency."""

    travel_agency_name: OptionalNullable[str] = UNSET
    r"""The name of the travel agency."""

    travel_agency_plan_name: OptionalNullable[str] = UNSET
    r"""The name of the travel agency plan."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "booking_code",
            "is_cardholder_traveling",
            "issued_address",
            "issued_at",
            "issuing_carrier_code",
            "issuing_carrier_name",
            "issuing_iata_designator",
            "issuing_icao_code",
            "legs",
            "passenger_name_record",
            "passengers",
            "reservation_system",
            "restricted_ticket",
            "ticket_delivery_method",
            "ticket_number",
            "travel_agency_code",
            "travel_agency_invoice_number",
            "travel_agency_name",
            "travel_agency_plan_name",
        ]
        nullable_fields = [
            "booking_code",
            "is_cardholder_traveling",
            "issued_address",
            "issued_at",
            "issuing_carrier_code",
            "issuing_carrier_name",
            "issuing_iata_designator",
            "issuing_icao_code",
            "legs",
            "passenger_name_record",
            "passengers",
            "reservation_system",
            "restricted_ticket",
            "ticket_delivery_method",
            "ticket_number",
            "travel_agency_code",
            "travel_agency_invoice_number",
            "travel_agency_name",
            "travel_agency_plan_name",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .taxidkind import TaxIDKind
from gr4vy.types import BaseModel
from gr4vy.utils import validate_open_enum
from pydantic.functional_validators import PlainValidator
from typing_extensions import Annotated, TypedDict


class TaxIDTypedDict(TypedDict):
    value: str
    r"""The tax ID for the buyer."""
    kind: TaxIDKind


class TaxID(BaseModel):
    value: str
    r"""The tax ID for the buyer."""

    kind: Annotated[TaxIDKind, PlainValidator(validate_open_enum(False))]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing_extensions import Annotated, NotRequired, TypedDict


class GivingBlockOptionsTypedDict(TypedDict):
    default_cryptocurrency: NotRequired[Nullable[str]]
    r"""The default cryptocurrency to present at checkout. This can be used to ensure the user is presented with the same currency in both your checkout and the Giving Block checkout."""


class GivingBlockOptions(BaseModel):
    default_cryptocurrency: Annotated[
        OptionalNullable[str], pydantic.Field(alias="defaultCryptocurrency")
    ] = UNSET
    r"""The default cryptocurrency to present at checkout. This can be used to ensure the user is presented with the same currency in both your checkout and the Giving Block checkout."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["defaultCryptocurrency"]
        nullable_fields = ["defaultCryptocurrency"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .producttype import ProductType
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import List
from typing_extensions import Annotated, NotRequired, TypedDict


class CartItemTypedDict(TypedDict):
    name: str
    r"""The name of the cart item. The value you set for this property may be truncated if the maximum length accepted by a payment service provider is less than 255 characters."""
    quantity: int
    r"""The quantity of this item in the cart. This value cannot be negative or zero."""
    unit_amount: int
    r"""The amount for an individual item represented as a monetary amount in the smallest currency unit for the given currency, for example `1299` USD cents represents `$12.99`. The amount sent through to the payment processor as unitary amount will be calculated to include the discount and tax values sent as part of this cart item."""
    discount_amount: NotRequired[Nullable[int]]
    r"""The amount discounted for this item represented as a monetary amount in the smallest currency unit for the given currency, for example `1299` USD cents represents `$12.99`."""
    tax_amount: NotRequired[Nullable[int]]
    r"""The tax amount for this item represented as a monetary amount in the smallest currency unit for the given currency, for example `1299` USD cents represents `$12.99`."""
    external_identifier: NotRequired[Nullable[str]]
    r"""An external identifier for the cart item. This can be set to any value and is not sent to the payment service."""
    sku: NotRequired[Nullable[str]]
    r"""The SKU or product code for the item."""
    upc: NotRequired[Nullable[str]]
    r"""The UPC for the item."""
    product_url: NotRequired[Nullable[str]]
    r"""The product URL for the item."""
    image_url: NotRequired[Nullable[str]]
    r"""The URL for the image of the item."""
    categories: NotRequired[Nullable[List[str]]]
    r"""A list of strings containing product categories for the item."""
    product_type: NotRequired[Nullable[ProductType]]
    r"""The product type of the cart item."""
    seller_country: NotRequired[Nullable[str]]
    r"""The seller country of the cart item."""
    tax_exempt: NotRequired[Nullable[bool]]
    r"""Whether the item is exempt of tax."""
    unit_of_measure: NotRequired[Nullable[str]]
    r"""The unit of measure or the unit of measure code."""
    commodity_code: NotRequired[Nullable[str]]
    r"""Item commodity code. Generally a UNSPSC code."""
    description: NotRequired[Nullable[str]]
    r"""Brief item description."""
    duty_amount: NotRequired[Nullable[int]]
    r"""Item import or export duties represented as a monetary amount in the smallest currency unit for the given currency, for example `1299` cents to create an authorization for `$12.99`"""
    shipping_amount: NotRequired[Nullable[int]]
    r"""Freight/shipping amount represented as a monetary amount in the smallest currency unit for the given currency, for example `1299` cents to create an authorization for `$12.99`"""


class CartItem(BaseModel):
    name: str
    r"""The name of the cart item. The value you set for this property may be truncated if the maximum length accepted by a payment service provider is less than 255 characters."""

    quantity: int
    r"""The quantity of this item in the cart. This value cannot be negative or zero."""

    unit_amount: int
    r"""The amount for an individual item represented as a monetary amount in the smallest currency unit for the given currency, for example `1299` USD cents represents `$12.99`. The amount sent through to the payment processor as unitary amount will be calculated to include the discount and tax values sent as part of this cart item."""

    discount_amount: OptionalNullable[int] = UNSET
    r"""The amount discounted for this item represented as a monetary amount in the smallest currency unit for the given currency, for example `1299` USD cents represents `$12.99`."""

    tax_amount: OptionalNullable[int] = UNSET
    r"""The tax amount for this item represented as a monetary amount in the smallest currency unit for the given currency, for example `1299` USD cents represents `$12.99`."""

    external_identifier: OptionalNullable[str] = UNSET
    r"""An external identifier for the cart item. This can be set to any value and is not sent to the payment service."""

    sku: OptionalNullable[str] = UNSET
    r"""The SKU or product code for the item."""

    upc: OptionalNullable[str] = UNSET
    r"""The UPC for the item."""

    product_url: OptionalNullable[str] = UNSET
    r"""The product URL for the item."""

    image_url: OptionalNullable[str] = UNSET
    r"""The URL for the image of the item."""

    categories: OptionalNullable[List[str]] = UNSET
    r"""A list of strings containing product categories for the item."""

    product_type: Annotated[
        OptionalNullable[ProductType], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""The product type of the cart item."""

    seller_country: OptionalNullable[str] = UNSET
    r"""The seller country of the cart item."""

    tax_exempt: OptionalNullable[bool] = UNSET
    r"""Whether the item is exempt of tax."""

    unit_of_measure: OptionalNullable[str] = UNSET
    r"""The unit of measure or the unit of measure code."""

    commodity_code: OptionalNullable[str] = UNSET
    r"""Item commodity code. Generally a UNSPSC code."""

    description: OptionalNullable[str] = UNSET
    r"""Brief item description."""

    duty_amount: OptionalNullable[int] = UNSET
    r"""Item import or export duties represented as a monetary amount in the smallest currency unit for the given currency, for example `1299` cents to create an authorization for `$12.99`"""

    shipping_amount: OptionalNullable[int] = UNSET
    r"""Freight/shipping amount represented as a monetary amount in the smallest currency unit for the given currency, for example `1299` cents to create an authorization for `$12.99`"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "discount_amount",
            "tax_amount",
            "external_identifier",
            "sku",
            "upc",
            "product_url",
            "image_url",
            "categories",
            "product_type",
            "seller_country",
            "tax_exempt",
            "unit_of_measure",
            "commodity_code",
            "description",
            "duty_amount",
            "shipping_amount",
        ]
        nullable_fields = [
            "discount_amount",
            "tax_amount",
            "external_identifier",
            "sku",
            "upc",
            "product_url",
            "image_url",
            "categories",
            "product_type",
            "seller_country",
            "tax_exempt",
            "unit_of_measure",
            "commodity_code",
            "description",
            "duty_amount",
            "shipping_amount",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
